@@ -272,8 +272,8 @@ impl<'a> Compiler<'a> {
             Ok(i) => i,
             Err(_) => return Err(self.error(CompilationErrorPayload::TooManyCards(il))),
         };
-        // at runtime: pop arguments in the same order as the variables were declared
-        for param in arguments.iter() {
+        // at runtime: pop arguments reverse order as the variables were declared
+        for param in arguments.iter().rev() {
             self.add_local(param.as_str())?;
         }
         for (ic, card) in cards.iter().enumerate() {
@@ -313,7 +313,7 @@ impl<'a> Compiler<'a> {
         Ok(())
     }
 
-    fn encode_jump(&mut self, lane: &LaneNode) -> CompilationResult<()> {
+    fn encode_jump(&mut self, lane: &str) -> CompilationResult<()> {
         let to = self.lookup_lane(&self.jump_table, lane)?;
         write_to_vec(to.hash_key, &mut self.program.bytecode);
         write_to_vec(to.arity, &mut self.program.bytecode);
@@ -324,10 +324,10 @@ impl<'a> Compiler<'a> {
     fn lookup_lane<'b>(
         &self,
         jump_table: &'b CaoHashMap<String, LaneMeta>,
-        lane: &LaneNode,
+        lane: &str,
     ) -> CompilationResult<&'b LaneMeta> {
         // attempt to look up the function by name
-        let mut to = jump_table.get(&lane.0);
+        let mut to = jump_table.get(lane);
         if to.is_none() {
             // attempt to look up the function in the current namespace
 
@@ -336,7 +336,7 @@ impl<'a> Compiler<'a> {
                 .current_namespace
                 .iter()
                 .flat_map(|x| [x.as_ref(), "."])
-                .chain(std::iter::once(lane.0.as_str()))
+                .chain(std::iter::once(lane))
                 .collect::<String>();
 
             to = jump_table.get(&name);
@@ -346,7 +346,7 @@ impl<'a> Compiler<'a> {
             if let Some((_, alias)) = self
                 .current_imports
                 .iter()
-                .find(|(import, _)| *import == &lane.0)
+                .find(|(import, _)| *import == lane)
             {
                 let (super_depth, suffix) = super_depth(alias);
                 let name = self
@@ -362,7 +362,7 @@ impl<'a> Compiler<'a> {
         }
         if to.is_none() {
             // attempt to look up by imported module
-            if let Some((prefix, suffix)) = lane.0.as_str().split_once('.') {
+            if let Some((prefix, suffix)) = lane.split_once('.') {
                 if let Some((_, alias)) = self
                     .current_imports
                     .iter()
@@ -385,7 +385,7 @@ impl<'a> Compiler<'a> {
         }
         to.ok_or_else(|| {
             self.error(CompilationErrorPayload::InvalidJump {
-                dst: lane.clone(),
+                dst: lane.to_string(),
                 msg: None,
             })
         })
@@ -430,13 +430,14 @@ impl<'a> Compiler<'a> {
                     self.current_index.pop_subindex()
                 }
             }
-            Card::ForEach {
-                i,
-                k,
-                v,
-                variable,
-                body,
-            } => {
+            Card::ForEach(fe) => {
+                let ForEach {
+                    i,
+                    k,
+                    v,
+                    variable,
+                    body,
+                } = fe.as_ref();
                 self.current_index.push_subindex(0);
                 self.process_card(variable)?;
                 self.current_index.pop_subindex();
@@ -446,15 +447,15 @@ impl<'a> Compiler<'a> {
                 let loop_item = self.add_local_unchecked("")?;
                 // ForEach instruction will push these values on the stack
                 let v = match v {
-                    Some(o) => self.add_local(&o.0)?,
+                    Some(o) => self.add_local(&o)?,
                     None => self.add_local_unchecked("")?,
                 };
                 let k = match k {
-                    Some(k) => self.add_local(&k.0)?,
+                    Some(k) => self.add_local(&k)?,
                     None => self.add_local_unchecked("")?,
                 };
                 let i = match i {
-                    Some(i) => self.add_local(&i.0)?,
+                    Some(i) => self.add_local(&i)?,
                     None => self.add_local_unchecked("")?,
                 };
                 self.push_instruction(Instruction::BeginForEach);
@@ -502,7 +503,7 @@ impl<'a> Compiler<'a> {
                 let loop_counter_index = self.add_local_unchecked("")?;
                 let i_index = match i {
                     Some(var) => {
-                        let index = self.add_local(&var.0)?;
+                        let index = self.add_local(&var)?;
                         Some(index)
                     }
                     None => None,
@@ -542,19 +543,19 @@ impl<'a> Compiler<'a> {
                 self.read_var_card(variable)?;
             }
             Card::SetVar(var) => {
-                let index = match self.resolve_var(var.0.as_str())? {
+                let index = match self.resolve_var(var.as_str())? {
                     Some(i) => i as u32,
-                    None => self.add_local(&var.0)?,
+                    None => self.add_local(&var)?,
                 };
 
                 self.write_local_var(index);
             }
             Card::SetGlobalVar(variable) => {
                 let next_var = &mut self.next_var;
-                if variable.0.is_empty() {
+                if variable.is_empty() {
                     return Err(self.error(CompilationErrorPayload::EmptyVariable));
                 }
-                let varhash = Handle::from_bytes(variable.0.as_bytes());
+                let varhash = Handle::from_bytes(variable.as_bytes());
 
                 let id = self
                     .program
@@ -570,7 +571,7 @@ impl<'a> Compiler<'a> {
                     .variables
                     .names
                     .entry(Handle::from_u32(id.0))
-                    .or_insert_with(move || *variable.0);
+                    .or_insert_with(move || variable.to_string());
                 write_to_vec(*id, &mut self.program.bytecode);
             }
             Card::IfElse(children) => {
@@ -606,7 +607,7 @@ impl<'a> Compiler<'a> {
                 self.current_index.pop_subindex();
             }
             Card::Jump(jmp) => self.encode_jump(jmp)?,
-            Card::StringLiteral(c) => self.push_str(c.0.as_str()),
+            Card::StringLiteral(c) => self.push_str(c.as_str()),
             Card::CallNative(c) => {
                 let name = &c.0;
                 let key = Handle::from_str(name.as_str()).unwrap();
@@ -645,8 +646,8 @@ impl<'a> Compiler<'a> {
         Ok(())
     }
 
-    fn read_var_card(&mut self, variable: &VarNode) -> CompilationResult<()> {
-        let scope = self.resolve_var(variable.0.as_str())?;
+    fn read_var_card(&mut self, variable: &str) -> CompilationResult<()> {
+        let scope = self.resolve_var(variable)?;
         match scope {
             Some(index) => {
                 //local
@@ -655,7 +656,7 @@ impl<'a> Compiler<'a> {
             None => {
                 // global
                 let next_var = &mut self.next_var;
-                let varhash = Handle::from_bytes(variable.0.as_bytes());
+                let varhash = Handle::from_bytes(variable.as_bytes());
                 let id = self
                     .program
                     .variables
@@ -671,7 +672,7 @@ impl<'a> Compiler<'a> {
                     .variables
                     .names
                     .entry(Handle::from_u32(id.0))
-                    .or_insert_with(|| *variable.0);
+                    .or_insert_with(|| variable.to_string());
                 self.push_instruction(Instruction::ReadGlobalVar);
                 write_to_vec(id, &mut self.program.bytecode);
             }
