@@ -1,5 +1,3 @@
-use std::fmt::Display;
-
 use super::*;
 use crate::InputString;
 use crate::VarName;
@@ -38,39 +36,49 @@ pub enum Card {
     GetProperty,
     ScalarInt(i64),
     ScalarFloat(f64),
-    StringLiteral(StringNode),
-    CallNative(Box<CallNode>),
+    StringLiteral(String),
+    CallNative(CallNode),
     IfTrue(Box<Card>),
     IfFalse(Box<Card>),
     /// Children = [then, else]
     IfElse(Box<[Card; 2]>),
-    Jump(LaneNode),
-    SetGlobalVar(VarNode),
-    SetVar(VarNode),
-    ReadVar(VarNode),
+    /// Lane name
+    Jump(String),
+    SetGlobalVar(VarName),
+    SetVar(VarName),
+    ReadVar(VarName),
     Repeat {
         /// Loop variable is written into this variable
-        i: Option<VarNode>,
+        i: Option<VarName>,
         body: Box<Card>,
     },
     /// Children = [condition, body]
     While(Box<[Card; 2]>),
-    // TODO: move the entire variant into a struct
-    ForEach {
-        /// Loop variable is written into this variable
-        i: Option<VarNode>,
-        /// The key is written into this variable
-        k: Option<VarNode>,
-        /// The value is written into this variable
-        v: Option<VarNode>,
-        /// Variable that is iterated on
-        variable: Box<Card>,
-        body: Box<Card>,
-    },
+    ForEach(Box<ForEach>),
     /// Single card that decomposes into multiple cards
     CompositeCard(Box<CompositeCard>),
 }
 
+#[derive(Debug, Clone)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
+pub struct ForEach {
+    /// Loop variable is written into this variable
+    pub i: Option<VarName>,
+    /// The key is written into this variable
+    pub k: Option<VarName>,
+    /// The value is written into this variable
+    pub v: Option<VarName>,
+    /// Variable that is iterated on
+    pub variable: Box<Card>,
+    pub body: Box<Card>,
+}
+
+impl From<ForEach> for Card {
+    fn from(value: ForEach) -> Self {
+        Card::ForEach(Box::new(value))
+    }
+}
+
 impl Card {
     pub fn name(&self) -> &str {
         match self {
@@ -229,19 +237,20 @@ impl Card {
         }))
     }
 
-    pub fn set_var(s: &str) -> Result<Self, arrayvec::CapacityError> {
-        let c = Self::SetVar(VarNode(Box::new(arrayvec::ArrayString::from_str(s)?)));
-        Ok(c)
+    pub fn set_var(s: impl Into<String>) -> Self {
+        Self::SetVar(s.into())
     }
 
-    pub fn read_var(s: &str) -> Result<Self, arrayvec::CapacityError> {
-        let c = Self::ReadVar(VarNode(Box::new(arrayvec::ArrayString::from_str(s)?)));
-        Ok(c)
+    pub fn call_native(s: impl Into<InputString>) -> Self {
+        Self::CallNative(CallNode(s.into()))
     }
 
-    pub fn set_global_var(s: &str) -> Result<Self, arrayvec::CapacityError> {
-        let c = Self::SetGlobalVar(VarNode(Box::new(arrayvec::ArrayString::from_str(s)?)));
-        Ok(c)
+    pub fn read_var(s: impl Into<String>) -> Self {
+        Self::ReadVar(s.into())
+    }
+
+    pub fn set_global_var(s: impl Into<String>) -> Self {
+        Self::SetGlobalVar(s.into())
     }
 
     pub fn scalar_int(i: i64) -> Self {
@@ -249,11 +258,11 @@ impl Card {
     }
 
     pub fn string_card(s: impl Into<String>) -> Self {
-        Self::StringLiteral(StringNode(s.into()))
+        Self::StringLiteral(s.into())
     }
 
     pub fn jump(s: impl Into<String>) -> Self {
-        Self::Jump(LaneNode(s.into()))
+        Self::Jump(s.into())
     }
 
     pub fn get_child_mut(&mut self, i: usize) -> Option<&mut Card> {
@@ -266,13 +275,14 @@ impl Card {
                 }
                 res = c;
             }
-            Card::ForEach {
-                i: _,
-                k: _,
-                v: _,
-                variable: a,
-                body: b,
-            } => {
+            Card::ForEach(fe) => {
+                let ForEach {
+                    i: _,
+                    k: _,
+                    v: _,
+                    variable: a,
+                    body: b,
+                } = fe.as_mut();
                 if i > 1 {}
                 match i {
                     0 => res = a.as_mut(),
@@ -296,13 +306,14 @@ impl Card {
                 }
                 res = c;
             }
-            Card::ForEach {
-                i: _,
-                k: _,
-                v: _,
-                variable: a,
-                body: b,
-            } => {
+            Card::ForEach(fe) => {
+                let ForEach {
+                    i: _,
+                    k: _,
+                    v: _,
+                    variable: a,
+                    body: b,
+                } = fe.as_ref();
                 if i > 1 {}
                 match i {
                     0 => res = a.as_ref(),
@@ -332,13 +343,14 @@ impl Card {
                 res = std::mem::replace::<Card>(c.as_mut(), Card::Pass);
             }
 
-            Card::ForEach {
-                i: _,
-                k: _,
-                v: _,
-                variable: a,
-                body: b,
-            } => {
+            Card::ForEach(fe) => {
+                let ForEach {
+                    i: _,
+                    k: _,
+                    v: _,
+                    variable: a,
+                    body: b,
+                } = fe.as_mut();
                 if i > 1 {}
                 match i {
                     0 => res = std::mem::replace::<Card>(a.as_mut(), Card::Pass),
@@ -376,13 +388,14 @@ impl Card {
                 *c.as_mut() = card;
             }
 
-            Card::ForEach {
-                i: _,
-                k: _,
-                v: _,
-                variable: a,
-                body: b,
-            } => {
+            Card::ForEach(fe) => {
+                let ForEach {
+                    i: _,
+                    k: _,
+                    v: _,
+                    variable: a,
+                    body: b,
+                } = fe.as_mut();
                 if i > 1 {}
                 match i {
                     0 => *a.as_mut() = card,
@@ -413,17 +426,20 @@ impl Card {
                 }
                 std::mem::replace(c.as_mut(), card)
             }
-            Card::ForEach {
-                i: _,
-                k: _,
-                v: _,
-                variable: a,
-                body: b,
-            } => match i {
-                0 => std::mem::replace(a.as_mut(), card),
-                1 => std::mem::replace(b.as_mut(), card),
-                _ => return Err(card),
-            },
+            Card::ForEach(fe) => {
+                let ForEach {
+                    i: _,
+                    k: _,
+                    v: _,
+                    variable: a,
+                    body: b,
+                } = fe.as_mut();
+                match i {
+                    0 => std::mem::replace(a.as_mut(), card),
+                    1 => std::mem::replace(b.as_mut(), card),
+                    _ => return Err(card),
+                }
+            }
             Card::While(children) | Card::IfElse(children) => {
                 let Some(c) = children.get_mut(i) else {
                     return Err(card);
@@ -440,49 +456,6 @@ impl Card {
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub struct CallNode(pub InputString);
 
-#[derive(Debug, Clone, Default)]
-#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
-pub struct SubProgramNode(pub InputString);
-
-#[derive(Debug, Clone, Default)]
-#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
-pub struct StringNode(pub String);
-
-#[derive(Debug, Clone, Default)]
-#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
-pub struct VarNode(pub Box<VarName>);
-
-impl VarNode {
-    /// panics if the string is too long
-    pub fn from_str_unchecked(s: &str) -> Self {
-        Self(Box::new(
-            VarName::from(s).expect("Failed to parse variable name"),
-        ))
-    }
-}
-
-#[derive(Debug, Clone, Default, PartialEq, Eq, Hash)]
-#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
-pub struct LaneNode(pub String);
-
-impl Display for LaneNode {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(f, "{}", self.0)
-    }
-}
-
-impl From<LaneNode> for Handle {
-    fn from(ln: LaneNode) -> Self {
-        Self::from(&ln)
-    }
-}
-
-impl<'a> From<&'a LaneNode> for Handle {
-    fn from(ln: &'a LaneNode) -> Self {
-        Handle::from_str(ln.0.as_str()).unwrap()
-    }
-}
-
 #[derive(Debug, Clone, Default)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub struct CompositeCard {
@@ -490,3 +463,9 @@ pub struct CompositeCard {
     pub ty: String,
     pub cards: Vec<Card>,
 }
+
+impl From<CompositeCard> for Card {
+    fn from(value: CompositeCard) -> Self {
+        Card::CompositeCard(Box::new(value))
+    }
+}
