@@ -1,3 +1,5 @@
+use crate::VarName;
+
 use super::*;
 
 #[test]
@@ -17,17 +19,15 @@ fn module_bincode_serde_test() {
 }
 
 fn prog() -> Module {
-    use crate::compiler::StringNode;
-
     let lanes = vec![(
         "main".into(),
         Lane::default().with_card(Card::CompositeCard(Box::new(
             crate::compiler::CompositeCard {
                 ty: "".to_string(),
                 cards: vec![
-                    Card::StringLiteral(StringNode("poggers".to_owned())),
-                    Card::StringLiteral(StringNode("poggers".to_owned())),
-                    Card::StringLiteral(StringNode("poggers".to_owned())),
+                    Card::StringLiteral("poggers".to_owned()),
+                    Card::StringLiteral("poggers".to_owned()),
+                    Card::StringLiteral("poggers".to_owned()),
                 ],
             },
         ))),
@@ -92,17 +92,15 @@ fn module_card_fetch_test() {
 
 #[test]
 fn remove_card_from_compositve_test() {
-    use crate::compiler::StringNode;
-
     let lanes = vec![(
         "main".into(),
         Lane::default().with_card(Card::CompositeCard(Box::new(
             crate::compiler::CompositeCard {
                 ty: "".to_string(),
                 cards: vec![
-                    Card::StringLiteral(StringNode("winnie".to_owned())),
-                    Card::StringLiteral(StringNode("pooh".to_owned())),
-                    Card::StringLiteral(StringNode("tiggers".to_owned())),
+                    Card::StringLiteral("winnie".to_owned()),
+                    Card::StringLiteral("pooh".to_owned()),
+                    Card::StringLiteral("tiggers".to_owned()),
                 ],
             },
         ))),
@@ -123,7 +121,7 @@ fn remove_card_from_compositve_test() {
         .unwrap();
 
     match removed {
-        Card::StringLiteral(s) => assert_eq!(s.0, "pooh"),
+        Card::StringLiteral(s) => assert_eq!(s, "pooh"),
         _ => panic!(),
     }
 }
@@ -153,7 +151,7 @@ fn remove_card_from_ifelse_test() {
         .unwrap();
 
     match removed {
-        Card::StringLiteral(s) => assert_eq!(s.0, "pooh"),
+        Card::StringLiteral(s) => assert_eq!(s, "pooh"),
         _ => panic!(),
     }
 
@@ -214,3 +212,104 @@ fn insert_card_test() {
 
     assert_eq!(json, EXP, "actual:\n{json}\nexpected:\n{EXP}");
 }
+
+#[test]
+fn lookup_jump_target_test() {
+    let mut program = CaoProgram::default();
+    program.submodules.push((
+        "foo".to_string(),
+        CaoProgram {
+            submodules: vec![(
+                "bar".to_string(),
+                CaoProgram {
+                    lanes: vec![(
+                        "poggers".to_string(),
+                        Lane {
+                            arguments: vec![
+                                VarName::from("winnie"),
+                                VarName::from("pooh"),
+                                VarName::from("tiggers"),
+                            ],
+                            cards: vec![],
+                        },
+                    )],
+                    ..Default::default()
+                },
+            )],
+            ..Default::default()
+        },
+    ));
+
+    let lane = program.lookup_lane("foo.bar.poggers").unwrap();
+
+    assert_eq!(
+        lane.arguments,
+        &[
+            VarName::from("winnie"),
+            VarName::from("pooh"),
+            VarName::from("tiggers"),
+        ]
+    );
+}
+
+#[test]
+fn lookup_jump_target_invalid_submodule_is_none_test() {
+    let mut program = CaoProgram::default();
+    program.submodules.push((
+        "foo".to_string(),
+        CaoProgram {
+            submodules: vec![(
+                "bar".to_string(),
+                CaoProgram {
+                    lanes: vec![(
+                        "poggers".to_string(),
+                        Lane {
+                            arguments: vec![
+                                VarName::from("winnie"),
+                                VarName::from("pooh"),
+                                VarName::from("tiggers"),
+                            ],
+                            cards: vec![],
+                        },
+                    )],
+                    ..Default::default()
+                },
+            )],
+            ..Default::default()
+        },
+    ));
+
+    let lane = program.lookup_lane("foo.baz.poggers");
+    assert!(lane.is_none());
+}
+
+#[test]
+fn lookup_jump_target_invalid_lane_is_none_test() {
+    let mut program = CaoProgram::default();
+    program.submodules.push((
+        "foo".to_string(),
+        CaoProgram {
+            submodules: vec![(
+                "bar".to_string(),
+                CaoProgram {
+                    lanes: vec![(
+                        "poggers".to_string(),
+                        Lane {
+                            arguments: vec![
+                                VarName::from("winnie"),
+                                VarName::from("pooh"),
+                                VarName::from("tiggers"),
+                            ],
+                            cards: vec![],
+                        },
+                    )],
+                    ..Default::default()
+                },
+            )],
+            ..Default::default()
+        },
+    ));
+
+    let lane = program.lookup_lane("foo.bar.poogers");
+    assert!(lane.is_none());
+}
